import re
import nltk
from nltk import bigrams
from nltk.probability import FreqDist
from sklearn.feature_extraction.text import TfidfVectorizer
import math
import copy
from text_extraction import extract_pf

def count_syllables(word):
    word = word.lower()
    syllable_count = len(re.findall(r'[aeiou]+', word))
    
    # Adjust for silent 'e' at the end (e.g., "cake")
    if word.endswith("e"):
        syllable_count -= 1
    return max(syllable_count, 1)

class Compute_Metrics(object):
    
    def __init__(self):
        nltk.download('punkt_tab')
        corpus = extract_pf("201", False)
        corpus_items = list(corpus[1].items())
        corpus_dict = dict(corpus_items[10:])
        self.corpus_strings = list(corpus_dict.values())
        
        # PMI FREQUENCSIES
        tokens = [word.lower() for text in self.corpus_strings for word in text.split()]
        bigram_list = list(bigrams(tokens))

        self.unigram_freq = FreqDist(tokens)
        self.bigram_freq = FreqDist(bigram_list)

        self.total_words = sum(self.unigram_freq.values())
        self.total_bigrams = sum(self.bigram_freq.values())
        
        # TF-IDF VALUES
        self.vectorizer = TfidfVectorizer()
      
    @staticmethod  
    def compute_ttr(text) -> float:
        words = text.lower().split()
        unique_words = set(words)
        return len(unique_words) / len(words)

    @staticmethod
    def compute_fre(text) -> float:
        words = re.findall(r"[A-Za-z]+(?:'[A-Za-z]+)?", text)
        sentences = re.split(r"(?<=[.?!])\s+", text)
        
        total_words = len(words)
        total_sentences = len(sentences)
        total_syllables = sum(count_syllables(word) for word in words)
        
        fre_score = 206.835 - 1.015 * (total_words / total_sentences) - 84.6 * (total_syllables / total_words)
        return fre_score

    def compute_pmi(self, text) -> dict[tuple, float]:
        pmi_scores = {}
        # word1 is first word, word2 is word after word1
        for idx, word1 in enumerate(text.split()):
            if idx < len(text.split()) - 1:
                word2 = text.split()[idx + 1]
                p_w1 = self.unigram_freq[word1] / self.total_words
                p_w2 = self.unigram_freq[word2] / self.total_words
                
                if p_w1 == 0 or p_w2 == 0:
                    pmi_scores[word1, word2] = 0
                    continue
                
                p_w1_w2 = self.bigram_freq[(word1, word2)] / self.total_bigrams if (word1, word2) in self.bigram_freq else 0.00001
                
                pmi = math.log2(p_w1_w2 / (p_w1 * p_w2))
                pmi_scores[word1, word2] = pmi

        sorted_pmi_scores = dict(sorted(pmi_scores.items(), key=lambda item: item[1], reverse=True))
        return sorted_pmi_scores

    def compute_tfidf(self, new_text) -> dict[str, float]:
        corpus = copy.deepcopy(self.corpus_strings)
        corpus.append(new_text)
        new_corpus = corpus
        tfidf_matrix = self.vectorizer.fit_transform(new_corpus)
        tfidf_new_text = tfidf_matrix[-1]
        
        tfidf_scores = tfidf_new_text.toarray().flatten()
        new_text_tokens = new_text.lower().split()
        feature_names = self.vectorizer.get_feature_names_out()
        # filter through so you only have indices of words in new_text
        feature_indices = [i for i, word in enumerate(feature_names) if word in new_text_tokens]
        tfidf_dict = {feature_names[i]: tfidf_scores[i] for i in feature_indices}
        
        sorted_tfidf = dict(sorted(tfidf_dict.items(), key=lambda item: item[1], reverse=True))
        
        return sorted_tfidf

def main():
    test_text = """I have been dreaming of this day for years, when the sky is blue. But now I'm tired enough to see it coming up out there in my dreams as if they were some kind thing that was born from nothing but sunlight…
    """
    Corpus = Compute_Metrics()
    pmi = Corpus.compute_pmi(test_text)
    ttr = Corpus.compute_ttr(test_text)
    fre = Corpus.compute_fre(test_text)
    tfidf = Corpus.compute_tfidf(test_text)

if __name__ == '__main__':
    main()